/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│ vi: set et ft=c ts=2 sts=2 sw=2 fenc=utf-8                               :vi │
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2020 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/elf/def.h"
#include "libc/elf/elf.h"
#include "libc/elf/struct/ehdr.h"
#include "libc/elf/struct/shdr.h"
#include "libc/str/str.h"

/**
 * Returns pointer to elf string table.
 *
 * @param elf points to the start of the executable image data
 * @param mapsize is the number of bytes past `elf` we can access
 * @param section_name is usually `".strtab"`, `".dynstr"`, or null
 * @return pointer to string table within `elf` image, which should
 *     normally be a sequence of NUL-terminated strings whose first
 *     string is the empty string; otherwise NULL is returned, when
 *     either: (1) `section_name` is not found, (2) it did not have
 *     the `SHT_STRTAB` section type, (3) the section size was zero
 *     noting that the ELF spec does consider that legal, or lastly
 *     (4) an overflow or boundary violation occurred
 */
char *GetElfStringTable(const Elf64_Ehdr *elf,  //
                        size_t mapsize,         //
                        const char *section_name) {
  int i;
  char *name;
  const Elf64_Shdr *shdr;
  for (i = 0; i < elf->e_shnum; ++i) {
    if ((shdr = GetElfSectionHeaderAddress(elf, mapsize, i)) &&
        shdr->sh_type == SHT_STRTAB &&
        (name = GetElfSectionName(elf, mapsize, shdr)) &&
        (!section_name || !strcmp(name, section_name))) {
      return GetElfSectionAddress(elf, mapsize, shdr);
    }
  }
  return 0;
}
